﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Effects;
using System.Windows.Threading;

namespace BarrageTransporter
{
    class BarrageHelper
    {
    }

    public class ShadowText : TextBlock
    {
        // TODO: 用影子优化动画的性能。
        public ShadowText(string text)
        {
            Text = text;
            FontSize = Properties.Settings.Default.barrageFontSize;
            Foreground = Brushes.White;

            Effect = new DropShadowEffect
            {
                Color = Colors.Black,
                BlurRadius = 2,
                ShadowDepth = 1,
                Opacity = 1,
                RenderingBias = RenderingBias.Performance
            };

            Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));
            Arrange(new Rect(0, 0, DesiredSize.Width, DesiredSize.Height));
        }
    }

    public class BarrageManager
    {
        private Grid container;

        private int lineHeight = Properties.Settings.Default.barrageFontSize + 6;
        private int paddingTop = 4;

        private Boolean[] isOccupy;
        private int lines;

        public BarrageManager(Grid grid)
        {
            container = grid;
            lines = (int)(container.RenderSize.Height / lineHeight) - 1;
            isOccupy = new Boolean[lines];
        }

        public int usableLine()
        {
            for (int line = 0; line < lines; line += 1)
            {
                if (!isOccupy[line])
                {
                    isOccupy[line] = true;
                    return line;
                }
            }
            return -1;
        }

        public void clearLine()
        {
            for (int line = 0; line < lines; line += 1)
            {
                isOccupy[line] = false;
            }
        }

        public int lineLocationY(int line)
        {
            return (line * lineHeight) + paddingTop;
        }

        public void Shoot(string text)
        {
            var line = usableLine();

            if (line == -1)
            {
                clearLine();
                line = usableLine();
            }

            // 初始化与显示
            FrameworkElement frameworkElement = new ShadowText(text);

            frameworkElement.Margin = new Thickness(0, lineLocationY(line), 0, 0);
            container.Children.Add(frameworkElement);

            // 初始化动画
            var anim = new DoubleAnimation();
            anim.From = this.container.RenderSize.Width;
            anim.To = -frameworkElement.DesiredSize.Width - 1;
            anim.SpeedRatio = frameworkElement.DesiredSize.Width > 80 ?
                (.05 * (frameworkElement.DesiredSize.Width / 1500 + 1)) :
                (.1 * ((100 - frameworkElement.DesiredSize.Width) / 100 + 1));
            TranslateTransform trans = new TranslateTransform();
            frameworkElement.RenderTransform = trans;

            // 末尾的处理
            anim.Completed += new EventHandler(delegate (Object o, EventArgs a)
            {
                container.Children.Remove(frameworkElement);
            });

            // 行管理
            var timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromMilliseconds(300);
            timer.Tick += new EventHandler(delegate (Object o, EventArgs a)
            {
                Point relativePoint = frameworkElement.TransformToAncestor(container).Transform(new Point(0, 0));
                if (relativePoint.X < container.ActualWidth - frameworkElement.DesiredSize.Width - 50)
                {
                    timer.Stop();
                    isOccupy[line] = false;
                }
            });
            timer.Start();

            // 播放动画
            trans.BeginAnimation(TranslateTransform.XProperty, anim);
        }

        public int BarrageCount
        {
            get
            {
                if (container == null)
                {
                    return 0;
                }
                else
                {
                    return container.Children.Count;
                }
            }
        }
    }
}
