﻿using AForge.Video.DirectShow;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Threading;
using WebSocketSharp;

namespace BarrageTransporter
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        private AppContext appContext = AppContext.getAppContext();
        private SerialTool serialTool = SerialTool.getSerialTool();

        SynchronizationContext synchronizationContext = null;
        private System.Timers.Timer timer;
        private bool barrageRunning = true;
        public WebSocket ws;

        private BarrageCurtainWindow curtain;
        private FilterInfoCollection videoDevices;
        public static Label status;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            status = portStatus;
            Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Background, new Action(() => {
                curtain = new BarrageCurtainWindow();
                curtain.Show();
                timer.Start();
            }));

            try
            {
                // 枚举所有视频输入设备
                videoDevices = new FilterInfoCollection(FilterCategory.VideoInputDevice);
                if (videoDevices.Count == 0)
                    throw new ApplicationException();

                foreach (FilterInfo device in videoDevices)
                {
                    cameras.Items.Add(device.Name);
                }

                cameras.SelectedIndex = 0;
            }
            catch (ApplicationException)
            {
                cameras.Items.Add("没有发现本地视频输入设备");
                videoDevices = null;
            }

            // 获取UI线程同步上下文
            synchronizationContext = SynchronizationContext.Current;

            ws = new WebSocket("wss://www.minbb.cn/blowing-apple");
            ws.OnMessage += (send, ee) =>
            {
                if (barrageRunning)
                {
                    JObject jo = (JObject)JsonConvert.DeserializeObject(ee.Data);
                    ThreadProcSafePost(jo["content"].ToString());
                }
            };
            ws.OnOpen += (send, ee) =>
            {
                Console.WriteLine("连接打开");
                networkStatus.Content = "已连接服务器";
                networkStatus.Foreground = new SolidColorBrush(Colors.Green);
            };
            ws.OnClose += (s, ee) =>
            {
                ws.Close();
                networkStatus.Content = "连接断开";
                networkStatus.Foreground = new SolidColorBrush(Colors.Red);
            };
            ws.OnError += (s, ee) =>
            {
                networkStatus.Content = "无网络连接";
                networkStatus.Foreground = new SolidColorBrush(Colors.Red);
            };
            ws.Connect();

            timer = new System.Timers.Timer();
            timer.Enabled = false;
            timer.AutoReset = true;             // 定时器自动执行
            timer.Interval = 500;              // 执行间隔时间,单位为毫秒
            timer.Elapsed += new System.Timers.ElapsedEventHandler(timerElapsedPause);

            if (Properties.Settings.Default.startOpenBarrage)
            {
                showBarrage.IsChecked = true;
            }
            loadDevice();
            // 监听 Windows 消息，获取窗口句柄一定要写在窗口 loaded 事件里，才能获取到窗口句柄，否则为空
            HwndSource hwndSource = PresentationSource.FromVisual(this) as HwndSource;  // 窗口过程
            if (hwndSource != null)
            {
                hwndSource.AddHook(new HwndSourceHook(DeveiceChanged)); // 挂钩
            }
        }

        private void ThreadProcSafePost(string text)
        {
            synchronizationContext.Post(SetTextSafePost, text);
        }

        private void SetTextSafePost(object text)
        {
            curtain.Shoot(text.ToString().Trim());
        }

        private void timerElapsedPause(object sender, System.Timers.ElapsedEventArgs e)
        {
            ThreadProcSafePost();
        }

        private void ThreadProcSafePost()
        {
            synchronizationContext.Post(CountSafePost, null);
        }

        private void CountSafePost(object obj)
        {
            int count = curtain.BarrageCount, send = 0;
            switch (count)
            {
                case 0:
                case 1:
                case 2:
                case 3:
                case 4:
                    send = 0;
                    break;
                case 5:
                case 6:
                case 7:
                    send = 1;
                    break;
                case 8:
                case 9:
                case 10:
                    send = 2;
                    break;
                case 11:
                case 12:
                case 13:
                    send = 3;
                    break;
                case 14:
                case 15:
                case 16:
                    send = 4;
                    break;
                case 17:
                case 18:
                case 19:
                    send = 5;
                    break;
                case 20:
                case 21:
                case 22:
                    send = 6;
                    break;
                case 23:
                case 24:
                case 25:
                    send = 7;
                    break;
                case 26:
                case 27:
                case 28:
                    send = 8;
                    break;
                case 29:
                case 30:
                default:
                    send = 9;
                    break;
            }
            level.Content = "#" + send;
            if (appContext.PortOpen)
            {
                serialTool.sendToPort(appContext.SerialPort, send.ToString());
                Console.WriteLine(DateTime.Now.ToLocalTime().ToString() + " -> " + send);
            }
        }

        private void connectCamera_Click(object sender, RoutedEventArgs e)
        {
            if (!sourcePlayer.IsRunning)
            {
                // 连接视频输入设备
                VideoCaptureDevice videoSource = new VideoCaptureDevice(videoDevices[cameras.SelectedIndex].MonikerString);
                videoSource.DesiredFrameSize = new System.Drawing.Size(1280, 960);
                videoSource.DesiredFrameRate = 1;
                sourcePlayer.VideoSource = videoSource;
                sourcePlayer.Start();
            }
        }

        private void closeCamera_Click(object sender, RoutedEventArgs e)
        {
            sourcePlayer.SignalToStop();
            sourcePlayer.WaitForStop();
        }

        private void showBarrage_Click(object sender, RoutedEventArgs e)
        {
            barrageRunning = (bool)showBarrage.IsChecked;
        }

        private void fullScreen_Click(object sender, RoutedEventArgs e)
        {
            toolBar.Visibility = Visibility.Hidden;
            Thickness thickness = new Thickness();
            thickness.Top = 0;
            videoPlayer.Margin = thickness;
            this.Topmost = false;
            this.WindowStyle = WindowStyle.None;
            this.WindowState = WindowState.Maximized;
        }

        private void settings_Click(object sender, RoutedEventArgs e)
        {
            new SettingsWindow().ShowDialog();
        }

        private void about_Click(object sender, RoutedEventArgs e)
        {
            new AboutBox().ShowDialog();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            MessageBoxResult messageBoxResult = MessageBox.Show("确定退出程序吗？", "提示", MessageBoxButton.OKCancel, MessageBoxImage.Question);
            if (messageBoxResult == MessageBoxResult.Cancel)
            {
                e.Cancel = true;
            }
            else
            {
                if (ws != null)
                {
                    ws.Close();
                }
                closeCamera_Click(null, null);
                barrageRunning = false;
                curtain.Close();
                Application.Current.Shutdown();
            }
        }

        private void Window_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == System.Windows.Input.Key.Escape)
            {
                toolBar.Visibility = Visibility.Visible;
                Thickness thickness = new Thickness();
                thickness.Top = 30;
                videoPlayer.Margin = thickness;
                this.WindowState = WindowState.Normal;
                this.WindowStyle = WindowStyle.SingleBorderWindow;
            }
        }

        private void loadDevice()
        {
            List<string> portList = serialTool.findPort();
            if (portList.Count == 0)
            {
                MessageBoxResult mbr = MessageBox.Show("没有找到可用硬件！\n请确保连接硬件后再打开本软件。", "错误提示", MessageBoxButton.OK, MessageBoxImage.Warning);
                if (mbr == MessageBoxResult.OK)
                {
                    this.Close();
                }
            }
            else if (portList.Count == 1)
            {
                SerialPort serialPort = serialTool.openPort(portList[0], "9600", "8", "1", "无", appContext.dataReceived);
                if (serialPort != null)
                {
                    appContext.SerialPort = serialPort;
                }
            }
            else
            {
                MessageBoxResult mbr = MessageBox.Show("请到设置页面配置正确串口！", "提示", MessageBoxButton.OK, MessageBoxImage.Information);
                if (mbr == MessageBoxResult.OK)
                {
                    new SettingsWindow().ShowDialog();
                }
            }
        }

        // Windows 消息编号，采用 Windows 的消息机制来捕获插入的 USB 状态
        public const int WM_DEVICE_CHANGE = 0x219;
        public const int DBT_DEVICEARRIVAL = 0x8000;
        public const int DBT_DEVICE_REMOVE_COMPLETE = 0x8004;

        // 设备插拔改变函数
        private IntPtr DeveiceChanged(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            if (msg == WM_DEVICE_CHANGE)
            {
                switch (wParam.ToInt32())
                {
                    case DBT_DEVICEARRIVAL:
                        // 设备插入
                        loadDevice();
                        Console.WriteLine("设备插入");
                        break;
                    case DBT_DEVICE_REMOVE_COMPLETE:
                        // 设备卸载
                        Console.WriteLine("设备卸载");
                        break;
                    default:
                        break;
                }
            }
            return IntPtr.Zero;
        }
    }
}
